from binderhub.repoproviders import RepoProvider
from datetime import timedelta, datetime, timezone
import json
import os
import time
import urllib.parse
import re
import subproces
from tornado import gen


def tokenize_spec(spec):
    """Tokenize spec into parts, error if spec invalid."""

    spec_parts = spec.split('/')
    if len(spec_parts) != 4:
        msg = 'Spec is not of the form "group/user/repo/git_commit_hash", provided: "{spec}".'.format(spec=spec)
        raise ValueError(msg)

    return spec_parts


class LocalRepoProvider(RepoProvider):
    """Provider for local git repos
    """

    def __init__(self):
        """Takes the spec and divides it up in smaller parts and puts it into self:
        <group>/<user>/<branch>
        """
        self.group, self.user, self.repo, self.git_commit_hash = tokenize_spec(self.spec)


    @gen.coroutine
    def get_resolved_ref(self):
        print("LocalRepoProvider.get_resolved_ref")
        self.git_commit_hash = spec


    async def get_resolved_spec(self):
        return self.get_repo_url()

    def get_repo_url(self):
        """
        While called repo URL, the return value of this function is passed
        as argument to repo2docker
        hence we return the spec as is.
        """
        return self.spec

    async def get_resolved_ref_url(self):
        """Return the URL of repository at this commit in history
        We do not provide a history, resolve to repo url
        """
        return self.get_repo_url()

    def get_build_slug(self):
        """
        return a unique id for each machine image that is built by repo2docker.
        a unique build slug
        The slug is used for caching?
        """
        return f'{self.project}-{self.git_commit_hash}'




import os

from setuptools import setup

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name='jupyter_openbis_authenticator',
    version='0.5a5',
    author='SIS | ID | ETH Zuerich',
    author_email='swen@ethz.ch',
    description='An authenticator for Jupyterhub which authenticates against openBIS.',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://sissource.ethz.ch/sispub/jupyter-openbis-integration/',
    license='Apache Software License Version 2.0',
    packages=['jupyter_openbis_authenticator'],
    install_requires=[
        'pytest',
        'sudospawner',
        'jupyterhub>=0.8.0',
        'python-git-info',
        'pybis>=1.15.1',
        'ldap3',
    ],
    entry_points={
        'jupyterhub.spawners': [
            'rrp_pbh_spawner = jupyter_openbis_authenticator.spawner:RRPPersistentBinderSpawner',
        ],
    },
    zip_safe=True
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-


import os
import sys
import pwd
import re
import time
#from persistent_bhub_config import PersistentBinderSpawner
from jupyterhub.spawner import LocalProcessSpawner
from traitlets import Unicode, Bool
import git
from git import Repo

def list_repos():
    return [{"path": repo} for repo in all_volumes()]

def all_volumes():
    projects_dir = "/var/rrp-projects"
    return (o for o in os.listdir(projects_dir)
                if os.path.isdir(os.path.join(projects_dir, o)))

def volumes(kube_spawner):
    """Builds a list of tuples (`display name`, `folder name`, `locked`)
    """
    pods = kube_spawner.pod_reflector.pods.values()
    consumed_vols = set(filter(None, (pod.metadata.annotations.get("rrp-vol")
                                      for pod in pods)))
    return ((vol, vol, vol in consumed_vols) for vol in all_volumes())


def get_lockfile(path):
    is_locked = False
    locked_by = ""
    locked_when = ""

    lockfile_path = os.path.join(path, '.lockfile')
    if os.path.isfile(lockfile_path):
        is_locked = True
        with open(lockfile_path, 'r') as fh:
            locked_by = fh.readline().strip()
        locked_when = time.ctime(os.path.getmtime(lockfile_path))

    return (is_locked, locked_by, locked_when)

def get_git_commit(path):
    try:
        repo = Repo(path)
    except git.NoSuchPathError:
        repo = None
        pass
    except git.InvalidGitRepositoryError:
        repo = None
        pass
        
    commit = repo.commit()
    git_commit = {
        "commit_hash": commit.hexsha,
        "commit_date": time.ctime(commit.committed_date),
        "commit_name": commit.committer.name,
        "commit_message": commit.message,
        "active_branch": repo.active_branch.name,
    }
    return git_commit


def vol_name(folder_name: str):
    """Volume names only may container lower case alphanumeric chars.
    To prevent collisions, we build a hex representation of the original
    name"""
    return "vol-" + folder_name.encode().hex().lower()

def get_project_dirs(project_dir):
    def _get_project_dirs():
        project_dirs = [] 
        for f in os.scandir(project_dir):
            if f.is_dir():
                (is_locked, locked_by, locked_when) = get_lockfile(f.path) 
                print('-------------------------------')
                print(f.path)
                git_commit = get_git_commit(f.path) or {}
                project_dirs.append(
                    {
                        "repo_url": f.path,
                        "display_name": f.name,
                        "image": "",
                        "ref": git_commit.get('commit_hash'),
                        "git_commit": git_commit,
                        "is_locked": is_locked,
                        "locked_by": locked_by,
                        "locked_when": locked_when,
                    }
                )
        return project_dirs
    return _get_project_dirs

class RRPPersistentBinderSpawner(LocalProcessSpawner):
    # the main project dir can be defined in the jupyterhub_config.py
    # c.RRPPersistentBinderSpawner.projects_dir = '/path/to/projects'

    projects_dir = Unicode(
        config=True,
        help='Directory of projects'
    )

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.user.project_dirs = get_project_dirs(self.projects_dir)
        return
        
    def get_state(self):
        _state = self.orm_spawner.state
        state = super().get_state()
        return state
